#include <proto/exec.h>
#include <exec/resident.h>
#include <exec/initializers.h>
#include <exec/execbase.h>
#include <exec/libraries.h>
#include <proto/dos.h>
#include <dos/dos.h>
#include <proto/intuition.h>
#include <graphics/gfxbase.h>

#define GradientSliderBase	GradientSliderBase_intern

#include "gradientslider_intern.h"

/****************************************************************************/

/* Reserved library-function and dummy entrypoint */
ULONG LIB_Reserved(void)
{
	return 0;
}

/****************************************************************************/

struct Library *LIB_Init(struct GradientSliderBase_intern *GradientSliderBase, BPTR SegList, struct ExecBase *sysbase);
struct Library *LIB_Open(void);
BPTR LIB_Close(void);
BPTR LIB_Expunge(void);
Class *LIB_ObtainClass(void);

/****************************************************************************/

static const char LibID[] = VERSION_STRING;

APTR LibVectors[] =
{
	(APTR) FUNCARRAY_32BIT_NATIVE,
	(APTR) LIB_Open,
	(APTR) LIB_Close,
	(APTR) LIB_Expunge,
	(APTR) LIB_Reserved,

	(APTR) LIB_ObtainClass,
	(APTR) -1L
};

ULONG LibInitTab[] =
{
	(ULONG) sizeof(struct GradientSliderBase_intern),
	(ULONG) LibVectors,
	(ULONG) NULL,
	(ULONG) LIB_Init
};

struct Resident ROMTag =     /* do not change */
{
	RTC_MATCHWORD,
	&ROMTag,
	&ROMTag + 1,
	RTF_PPC | RTF_AUTOINIT | RTF_EXTENDED,
	VERSION_NUMBER,
	NT_LIBRARY,
	0,
	NAME_STRING,
	&LibID[6],
	(APTR)&LibInitTab,
	REVISION_NUMBER,
	NULL
};

ULONG __abox__ = 1;

/****************************************************************************/

static void LibCleanup(struct GradientSliderBase_intern *GradientSliderBase)
{
	if (GfxBase) CloseLibrary((struct Library *)GfxBase);
	if (IntuitionBase) CloseLibrary((struct Library *)IntuitionBase);
	if (UtilityBase) CloseLibrary(UtilityBase);
	if (CyberGfxBase) CloseLibrary(CyberGfxBase);
	if (LayersBase) CloseLibrary(LayersBase);
}

struct Library	*LIB_Init(struct GradientSliderBase_intern *GradientSliderBase, BPTR SegList, struct ExecBase *sysbase)
{
	GradientSliderBase->library.lib_Revision = REVISION_NUMBER;
	SysBase = sysbase;
	GradientSliderBase->seglist = SegList;

	if ((GfxBase = OpenLibrary("graphics.library", 39)) &&
		(IntuitionBase = OpenLibrary("intuition.library", 39)) &&
		(LayersBase = OpenLibrary("layers.library", 39)) &&
		(UtilityBase = OpenLibrary("utility.library", 39)))
	{
		CyberGfxBase = OpenLibrary("cybergraphics.library", 40);

		if ((GradientSliderBase->classptr = InitGradientSliderClass(GradientSliderBase)))
		{
			return((struct Library *)GradientSliderBase);
		}
	}

	LibCleanup(GradientSliderBase);

	FreeMem((APTR)((ULONG)(GradientSliderBase) - (ULONG)(GradientSliderBase->library.lib_NegSize)),
			GradientSliderBase->library.lib_NegSize + GradientSliderBase->library.lib_PosSize);

	return(NULL);
}

/****************************************************************************/

struct Library *LIB_Open(void)
{
	struct GradientSliderBase_intern *GradientSliderBase = (struct GradientSliderBase_intern *)REG_A6;

	GradientSliderBase->library.lib_Flags &= ~LIBF_DELEXP;
	GradientSliderBase->library.lib_OpenCnt++;

	return((struct Library *)GradientSliderBase);
}

/****************************************************************************/

BPTR LibExpunge(struct GradientSliderBase_intern *GradientSliderBase)
{
	if ((GradientSliderBase->library.lib_OpenCnt == 0))
	{
		BPTR seglist;

		if (GradientSliderBase->classptr)
		{
			if (!FreeClass(GradientSliderBase->classptr))
			{
				AddClass(GradientSliderBase->classptr);
				return 0L;
			}
		}

		seglist = GradientSliderBase->seglist;
		Remove((struct Node *)GradientSliderBase);
		LibCleanup(GradientSliderBase);

		FreeMem((APTR)((ULONG)(GradientSliderBase) - (ULONG)(GradientSliderBase->library.lib_NegSize)),
				GradientSliderBase->library.lib_NegSize + GradientSliderBase->library.lib_PosSize);

		return seglist;
	} else GradientSliderBase->library.lib_Flags |= LIBF_DELEXP;

	return 0L;
}

/****************************************************************************/

BPTR LIB_Close(void)
{
	struct GradientSliderBase_intern *GradientSliderBase = (struct GradientSliderBase_intern *)REG_A6;

	if (GradientSliderBase->library.lib_OpenCnt > 0) GradientSliderBase->library.lib_OpenCnt--;

	if ((GradientSliderBase->library.lib_OpenCnt == 0) && (GradientSliderBase->library.lib_Flags & LIBF_DELEXP))
		return LibExpunge(GradientSliderBase);

	return 0L;
}

/****************************************************************************/

BPTR LIB_Expunge(void)
{
	struct GradientSliderBase_intern *GradientSliderBase = (struct GradientSliderBase_intern *)REG_A6;

	return LibExpunge(GradientSliderBase);
}


/****************************************************************************/

Class *LIB_ObtainClass(void)
{
	struct GradientSliderBase_intern *GradientSliderBase = (struct GradientSliderBase_intern *)REG_A6;

	return(GradientSliderBase->classptr);
}

/****************************************************************************/
